/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Connection.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.descriptor;

import javax.xml.namespace.QName;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.objectweb.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * The connection element declares a mapping from consumer-provided service
 * address to a provider's service endpoint address. This mapping applies to
 * consumption of services by all components. The consumer subelement is used to
 * declare the type of consumer address that is to be mapped (or <i>connected</i>).
 * This declaration can take two forms:
 * <ul>
 * <li><b>service type.</b> This specifies a service type, by supplying an
 * interface-name attribute. All applicable exchanges that are addressed to the
 * given service type are to be sent to the given service provider endpoint
 * instead.</li>
 * <li><b>service endpoint.</b> This specifies a service endpoint, by
 * supplying a service-name attribute and an endpoint-name attribute. All
 * applicable exchanges that are addressed to the given service endpoint are to
 * be sent to the given service provider endpoint instead.</li>
 * </ul>
 * The provider attributes is used to declare the service provider endpoint to
 * which the consumer is to be connected.
 *
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
@SuppressWarnings("PMD.UnnecessaryConstrucctor")
public class Connection {

    /**
     * Consumer Endpoint Name for a service endpoint connection.
     */
    private String consumerEndpointName;

    /**
     * Consumer Interface Name for a service type connection.
     */
    private QName consumerInterfaceName;

    /**
     * Consumer Service Name for a service endpoint connection.
     */
    private QName consumerServiceName;

    /**
     * Provider Endpoint Name to be mapped the connection.
     */
    private String providerEndpointName;

    /**
     * Provider Service Name to be mapped the connection.
     */
    private QName providerServiceName;

    /**
     * Default constructor.
     */
    public Connection() {
        super();
    }

    @Override
    @SuppressWarnings("PMD.CloseResource")
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof Connection) {
            final Connection castOther = (Connection) other;
            bRet = new EqualsBuilder().append(this.consumerInterfaceName,
                    castOther.consumerInterfaceName).append(this.consumerServiceName,
                    castOther.consumerServiceName).append(this.consumerEndpointName,
                    castOther.consumerEndpointName).append(this.providerServiceName,
                    castOther.providerServiceName).append(this.providerEndpointName,
                    castOther.providerEndpointName).isEquals();
        } else {
            bRet = false;
        }
        return bRet;
    }

    /**
     * Returns the consumer Interface Name for a <b>service type</b>
     * connection.
     *
     * @return <code>QName</code> the interface name.
     */
    public final QName getConsumerInterfaceName() {
        return this.consumerInterfaceName;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.consumerInterfaceName).append(
                this.consumerServiceName).append(this.consumerEndpointName).append(
                this.providerServiceName).append(this.providerEndpointName).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("consumerInterfaceName",
                this.consumerInterfaceName).append("consumerServiceName",
                this.consumerServiceName).append("consumerEndpointName",
                this.consumerEndpointName).append("providerServiceName",
                this.providerServiceName).append("providerEndpointName",
                this.providerEndpointName).toString();
    }

    /**
     * Generate a XML representation of the service assembly descriptor node
     *
     * @return the XML representation
     */
    public final Node getNode(final Document jbiDescriptorDocument) {

        final Element connectionElement = jbiDescriptorDocument
                .createElementNS(XMLUtil.JBI_NAMESPACE, "connection");

        // Consumer element
        final Element consumerElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "consumer");
        connectionElement.appendChild(consumerElement);

        final Element interfaceNameElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "interface-name");
        XMLUtil.setQNameValue(this.consumerInterfaceName, jbiDescriptorDocument, interfaceNameElement);
        consumerElement.appendChild(interfaceNameElement);

        final Element serviceNameConsumerElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "service-name");
        XMLUtil.setQNameValue(this.consumerServiceName, jbiDescriptorDocument, serviceNameConsumerElement);
        consumerElement.appendChild(serviceNameConsumerElement);

        final Element endpointNameConsumerElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "endpoint-name");
        endpointNameConsumerElement.setTextContent(this.consumerEndpointName);
        consumerElement.appendChild(endpointNameConsumerElement);

        // Provider element
        final Element providerElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "provider");
        connectionElement.appendChild(providerElement);

        final Element serviceNameProviderElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "service-name");
        XMLUtil.setQNameValue(this.providerServiceName, jbiDescriptorDocument, serviceNameProviderElement);
        consumerElement.appendChild(serviceNameProviderElement);

        final Element endpointNameProviderElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "endpoint-name");
        endpointNameProviderElement.setTextContent(this.providerEndpointName);
        consumerElement.appendChild(endpointNameProviderElement);

        return connectionElement;
    }

    //
    // Getters and Setters
    //

    /**
     * Returns the consumer Endpoint Name for a <b>service endpoint</b>
     * connection.
     *
     * @return <code>String</code> the endpoint name.
     */
    public final String getConsumerEndpointName() {
        return this.consumerEndpointName;
    }

    /**
     * Returns the consumer Service Name for a <b>service endpoint</b>
     * connection.
     *
     * @return <code>QName</code> the service name.
     */
    public final QName getConsumerServiceName() {
        return this.consumerServiceName;
    }

    /**
     * Returns the provider Endpoint Name which the connection is to be mapped.
     *
     * @return <code>String</code> the endpoint name.
     */
    public final String getProviderEndpointName() {
        return this.providerEndpointName;
    }

    /**
     * Returns the provider Service Name which the connection is to be mapped.
     *
     * @return <code>QName</code> the service name.
     */
    public final QName getProviderServiceName() {
        return this.providerServiceName;
    }

    /**
     * Set the consumer Endpoint Name for a <b>service endpoint</b> connection.
     *
     * @param endpointName
     *            the endpoint name.
     */
    public final void setConsumerEndpointName(final String endpointName) {
        this.consumerEndpointName = endpointName;
    }

    /**
     * Set the consumer Interface Name for a <b>service type</b> connection.
     *
     * @param interfaceName
     *            the interface name.
     */
    public final void setConsumerInterfaceName(final QName interfaceName) {
        this.consumerInterfaceName = interfaceName;
    }

    /**
     * Set the consumer Service Name for a <b>service endpoint</b> connection.
     *
     * @param serviceName
     *            the service name.
     */
    public final void setConsumerServiceName(final QName serviceName) {
        this.consumerServiceName = serviceName;
    }

    /**
     * Set the provider Endpoint Name to be mapped.
     *
     * @param endpointName
     *            the endpoint name.
     */
    public final void setProviderEndpointName(final String endpointName) {
        this.providerEndpointName = endpointName;
    }

    /**
     * Set the provder Service Name to be mapped.
     *
     * @param serviceName
     *            the service name.
     */
    public final void setProviderServiceName(final QName serviceName) {
        this.providerServiceName = serviceName;
    }

}
