/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.descriptor;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * The consumes element is used to declare a service consumed by a service unit.
 * A consumed service must be define with the following required attribute:
 * <ul>
 * <li><b>interface-name.</b> This qualified name indicates the type of
 * service to be consumed.</li>
 * </ul>
 * Optionally a fully qualified name of the endpoint the SU will use to consume
 * the service can be specified. If the service endpoint name is declared, an
 * optional attribute can be supplied:
 * <ul>
 * <li><b>link-type.</b> This attribute is used to indicate the expected use
 * of this consumer with connection links. The possible values of this
 * attributes are:</li>
 * <ul>
 * <li><b>standard.</b> This is the default value. This indicates that the
 * provided service endpoint must be routed according to standard normalized
 * message routing rules.</li>
 * <li><b>hard.</b>This indicates that the provided service endpoint name must
 * match a service provider's service endpoint name; indirect connections are
 * not allowed.</li>
 * <li><b>soft.</b>This indicates that the provided service endpoint must
 * <i>not</i> match a service provider's service endpoint name; rather, it must
 * match an indirect connection name.</li>
 * </ul>
 * </ul>
 *
 * @version $Rev: 477 $ $Date: 2006-05-29 15:18:07Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
@SuppressWarnings("PMD.UnnecessaryConstrucctor")
public class Consumes extends ServiceUnitExtensibleElement {

    /**
     * Hard Link Type.
     */
    public static final String HARD_LINK_TYPE = "hard";

    /**
     * Soft Link Type.
     */
    public static final String SOFT_LINK_TYPE = "soft";

    /**
     * Standard Link Type.
     */
    public static final String STD_LINK_TYPE = "standard";

    /**
     * The optional link type.
     */
    private String linkType;

    /**
     * Default constructor.
     */
    public Consumes() {
        super();
    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof Consumes) {
            final Consumes castOther = (Consumes) other;
            bRet = new EqualsBuilder().append(this.endpointName, castOther.endpointName)
                    .append(this.interfaceName, castOther.interfaceName).append(
                            this.linkType, castOther.linkType).append(this.serviceName,
                            castOther.serviceName).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.endpointName).append(this.interfaceName)
                .append(this.linkType).append(this.serviceName).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("endpointName", this.endpointName)
                .append("interfaceName", this.interfaceName).append("linkType",
                        this.linkType).append("serviceName", this.serviceName).toString();
    }

    /**
     * Returns the optional link type associated to a fully qualified endpoint
     * the SU will use to consume the service.
     *
     * @return <code>String</code> the link type: <code>"standard"</code>,
     *         <code>"hard"</code> or <code>"soft"</code>.
     */
    public final String getLinkType() {
        return this.linkType;
    }

    /**
     * Returns <code>true</code> if the hard link type is used.
     */
    public final boolean isHardLink() {
        return (this.hasQualifiedEndpoint())
                && (this.linkType != null && this.linkType.equals(Consumes.HARD_LINK_TYPE));
    }

    /**
     * Returns <code>true</code> if the soft link type is used.
     */
    public final boolean isSoftLink() {
        return (this.hasQualifiedEndpoint())
                && (this.linkType != null && this.linkType.equals(Consumes.SOFT_LINK_TYPE));
    }

    /**
     * Returns <code>true</code> if the standard link type is used.
     */
    public final boolean isStandardLink() {
        return (this.hasQualifiedEndpoint())
                && (this.linkType == null || this.linkType.equals(Consumes.STD_LINK_TYPE));
    }

    /**
     * Returns <code>true</code> if the optional fully qualified endpoint name
     * have been specified. Otherwise, returns <code>false</code>.
     */
    public final boolean hasQualifiedEndpoint() {
        return this.serviceName != null && this.endpointName != null;
    }

    /**
     * Set the optional link type.
     *
     * @param type
     *            the link type.
     */
    public final void setLinkType(final String type) {
        this.linkType = type;
    }

    /**
     * Method used by {@link #getNode(Document)} to get the name of the XML root node of this object.
     */
    @Override
    protected final String getNodeName() {
        return "consumes";
    }
}
