/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIDescriptor.java 477 2006-05-29 15:18:07Z rmarins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.descriptor;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.objectweb.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * JBI Descriptor.
 *
 * @version $Rev: 477 $ $Date: 2006-05-29 15:18:07Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
@SuppressWarnings("PMD.UnnecessaryConstrucctor")
public class JBIDescriptor {

    // ----------------------------------------------------------------------
    // constants and inner classes declarations
    // ----------------------------------------------------------------------

    /**
     * ClassLoader parent first delegation option (<code>"parent-first"</code>).
     */
    private static final String CL_PARENT_FIRST = "parent-first";

    /**
     * ClassLoader self first delegation option (<code>"self-first"</code>).
     */
    private static final String CL_SELF_FIRST = "self-first";

    /**
     * JBI Component production.
     */
    private ComponentDescription component;

    /**
     * Service Assembly production.
     */
    private ServiceAssembly serviceAssembly;

    /**
     * Services production.
     */
    private Services services;

    /**
     * Shared Library production.
     */
    private SharedLibrary sharedLibrary;

    /**
     * JBI Descriptor version.
     */
    private double version;

    /**
     * Default constructor.
     */
    public JBIDescriptor() {
        super();
    }

    /**
     * Return true if the delegation model is "parent-first" or null (default)
     * Return false otherwise
     */
    public static boolean isParentFirst(final String delegation) {
        return (delegation == null) || (JBIDescriptor.CL_PARENT_FIRST.equals(delegation));
    }

    /**
     * Return true if the delegation model is "self-first" Return false
     * otherwise
     */
    public static boolean isSelfFirst(final String delegation) {
        return JBIDescriptor.CL_SELF_FIRST.equals(delegation);

    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof JBIDescriptor) {
            final JBIDescriptor castOther = (JBIDescriptor) other;
            bRet = new EqualsBuilder().append(this.component, castOther.component)
                    .append(this.serviceAssembly, castOther.serviceAssembly).append(
                            this.services, castOther.services).append(this.sharedLibrary,
                            castOther.sharedLibrary).append(this.version,
                            castOther.version).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.component).append(this.serviceAssembly)
                .append(this.services).append(this.sharedLibrary).append(this.version)
                .toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("component", this.component).append(
                "serviceAssembly", this.serviceAssembly)
                .append("services", this.services).append("sharedLibrary",
                        this.sharedLibrary).append("version", this.version).toString();
    }

    /**
     * Generate a XML representation of the JBI descriptor
     * @return the XML respresentation as a String
     */
    public final String toXmlText() throws JBIDescriptorException {

        try {
            final DocumentBuilderFactory documentFactory = DocumentBuilderFactory.newInstance();
            documentFactory.setNamespaceAware(true);

            final DocumentBuilder documentBuilder = documentFactory.newDocumentBuilder();
            final Document jbiDescriptorDocument = documentBuilder.newDocument();

            final Element jbiElement = jbiDescriptorDocument.createElementNS(
                    XMLUtil.JBI_NAMESPACE, "jbi");

            final Attr versionAttribute = jbiDescriptorDocument.createAttribute("version");
            versionAttribute.setValue(Double.toString(this.version));
            jbiElement.setAttributeNode(versionAttribute);

            if (this.component != null) {
                jbiElement.appendChild(this.component.getNode(jbiDescriptorDocument));
            }
            if (this.serviceAssembly != null) {
                jbiElement.appendChild(this.serviceAssembly.getNode(jbiDescriptorDocument));
            }
            if (this.services != null) {
                jbiElement.appendChild(this.services.getNode(jbiDescriptorDocument));
            }
            if (this.sharedLibrary != null) {
                jbiElement.appendChild(this.sharedLibrary.getNode(jbiDescriptorDocument));
            }

            jbiDescriptorDocument.appendChild(jbiElement);

            return XMLUtil.createStringFromDOMDocument(jbiDescriptorDocument);
        } catch (final DOMException e) {
            throw new JBIDescriptorException(e);
        } catch (final ParserConfigurationException e) {
            throw new JBIDescriptorException(e);
        } catch (final TransformerException e) {
            throw new JBIDescriptorException(e);
        }
    }

    // ----------------------------------------------------------------------
    // Getters and Setters
    // ----------------------------------------------------------------------

    /**
     * Returns the component production loaded from JBI Descriptor.
     *
     * @return Returns the component.
     */
    public final ComponentDescription getComponent() {
        return this.component;
    }

    /**
     * Returns the service assembly production loaded from JBI Descriptor.
     *
     * @return Returns the serviceAssembly.
     */
    public final ServiceAssembly getServiceAssembly() {
        return this.serviceAssembly;
    }

    /**
     * Returns the services production loaded from JBI Descriptor.
     *
     * @return Returns the services.
     */
    public final Services getServices() {
        return this.services;
    }

    /**
     * Returns the shared library production from JBI Descriptor.
     *
     * @return Returns the sharedLibrary.
     */
    public final SharedLibrary getSharedLibrary() {
        return this.sharedLibrary;
    }

    /**
     * Attribute version is a decimal value indicating the version of descriptor
     * in use.
     *
     * @return <code>double</code> the version.
     */
    public final double getVersion() {
        return this.version;
    }

    /**
     * Set the component JBI Descriptor production loaded from a XML.
     *
     * @param component
     *            the component production.
     */
    public final void setComponent(final ComponentDescription component) {
        this.component = component;
    }

    /**
     * Set the service assembly JBI Descriptor production loaded from a XML.
     *
     * @param serviceAssembly
     *            the service assembly production.
     */
    public final void setServiceAssembly(final ServiceAssembly serviceAssembly) {
        this.serviceAssembly = serviceAssembly;
    }

    /**
     * Set the services JBI Descriptor production loaded from a XML.
     *
     * @param services
     *            the services production.
     */
    public final void setServices(final Services services) {
        this.services = services;
    }

    /**
     * Set the shared library JBI Descriptor production loaded from a XML.
     *
     * @param sharedLibrary
     *            the shared library production.
     */
    public final void setSharedLibrary(final SharedLibrary sharedLibrary) {
        this.sharedLibrary = sharedLibrary;
    }

    /**
     * Attribute version is a decimal value indicating the version of descriptor
     * in use.
     *
     * @param version
     *            The version <code>double</code> value to set.
     */
    public final void setVersion(final double version) {
        this.version = version;
    }
}
