/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.descriptor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.objectweb.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * The services element is used to declare the static services provided and
 * consumed by a service unit. The term "static" in this context means those
 * services that are provided or consumed due to the deployment of the service
 * unit, and that are known at design- and/or deployment-time.
 *
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class Services extends ExtensibleJbiElement {

    /**
     * Used to distinguish between service engine and binding component.
     */
    private boolean bindingComponent;

    /**
     * Services provided by a service unit.
     */
    private List<Provides> provides;

    /**
     * Services consumed by a service unit.
     */
    private List<Consumes> consumes;

    /**
     * Default constructor.
     */
    public Services() {
        super();
        this.consumes = new ArrayList<Consumes>();
        this.provides = new ArrayList<Provides>();
    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof Services) {
            final Services castOther = (Services) other;
            bRet = new EqualsBuilder().append(this.bindingComponent,
                    castOther.bindingComponent)
                    .append(this.provides, castOther.provides).append(this.consumes,
                            castOther.consumes).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.bindingComponent).append(this.provides)
                .append(this.consumes).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("bindingComponent",
                this.bindingComponent).append("provides", this.provides).append(
                "consumes", this.consumes).toString();
    }

    /**
     * Generate a XML representation of the services descriptor node
     *
     * @return the XML representation
     */
    @Override
    public Node getNode(final Document jbiDescriptorDocument) {

        final Element servicesElement = jbiDescriptorDocument
                .createElementNS(XMLUtil.JBI_NAMESPACE, "services");

        final Attr bindingAttribute = jbiDescriptorDocument.createAttributeNS(
                XMLUtil.JBI_NAMESPACE, "binding-component");
        bindingAttribute.setValue(Boolean.toString(this.bindingComponent));
        servicesElement.setAttributeNode(bindingAttribute);

        for (int i = 0; i < this.provides.size(); i++) {
            servicesElement.appendChild(this.provides.get(i).getNode(jbiDescriptorDocument));
        }

        for (int i = 0; i < this.consumes.size(); i++) {
            servicesElement.appendChild(this.consumes.get(i).getNode(jbiDescriptorDocument));
        }

        return servicesElement;
    }

    //
    // Getters and Setters
    //

    /**
     * Attribute used to distinguish if the services declared is related to a
     * service engine or a binding component.
     *
     * @return <code>boolean</code> indicates if it's a binding component.
     */
    public final boolean isBindingComponent() {
        return this.bindingComponent;
    }

    public final void addConsumes(final Consumes consumesElem) {
        this.consumes.add(consumesElem);
    }

    public final void addProvides(final Provides providesElem) {
        this.provides.add(providesElem);
    }

    /**
     * Set if it's relative to a binding component.
     *
     * @param bindingComponent
     *            <code>true</code> to indicate it's a binding component.
     */
    public final void setBindingComponent(final boolean bindingComponent) {
        this.bindingComponent = bindingComponent;
    }

    /**
     * Set the list of services consumed by a service unit deployed.
     *
     * @param consumes
     *            the list of consumed services.
     */
    public final void setConsumes(final List<Consumes> consumes) {
        this.consumes = consumes;
    }

    /**
     * Returns the list of services consumed by a service unit deployed.
     *
     * @return consumes the list of consumed services.
     */
    public final List<Consumes> getConsumes() {
        return this.consumes;
    }

    /**
     * Set the list of services provided with the deployment of a service unit.
     *
     * @param provides
     *            the list of provided services.
     */
    public final void setProvides(final List<Provides> provides) {
        this.provides = provides;
    }

    /**
     * Returns the list of services provided with the deployment of a service
     * unit.
     *
     * @return provides the list of provided services.
     */
    public final List<Provides> getProvides() {
        return this.provides;
    }

}
