/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibray.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.descriptor;

import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.objectweb.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Shared Library production is used to describe a shared library installation
 * package.
 *
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
@SuppressWarnings("PMD.UnnecessaryConstructor")
public class SharedLibrary {

    /**
     * Shared Library identification data.
     */
    private Identification identification;

    /**
     * Shared Library class loading delegation model: <code>"self-first"</code>
     * or <code>"parent-first"</code>.
     */
    private String classLoaderDelegation;

    /**
     * Optional shared library version.
     */
    private String version;

    /**
     * Shared Library Class Path elements.
     */
    private List<String> sharedLibraryClassPath;

    /**
     * Default constructor.
     */
    public SharedLibrary() {
        super();
    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof SharedLibrary) {
            final SharedLibrary castOther = (SharedLibrary) other;
            bRet = new EqualsBuilder().append(this.identification,
                    castOther.identification).append(this.classLoaderDelegation,
                    castOther.classLoaderDelegation).append(this.version,
                    castOther.version).append(this.sharedLibraryClassPath,
                    castOther.sharedLibraryClassPath).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.identification).append(
                this.classLoaderDelegation).append(this.version).append(
                        this.sharedLibraryClassPath).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("identification",
                this.identification).append("classLoaderDelegation",
                        this.classLoaderDelegation).append("version", this.version).append(
                "sharedLibraryClassPath", this.sharedLibraryClassPath).toString();
    }

    /**
     * Generate a XML representation of the service assembly descriptor node
     * @return the XML representation
     */
    public final Node getNode(final Document jbiDescriptorDocument) {

        final Element sharedLibraryElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "shared-library");

        if (this.classLoaderDelegation != null
                && !"".equals(this.classLoaderDelegation)) {
            final Attr classLoaderDelegationAttribute = jbiDescriptorDocument
                    .createAttributeNS(XMLUtil.JBI_NAMESPACE,
                            "class-loader-delegation");
            classLoaderDelegationAttribute.setValue(this.classLoaderDelegation);
            sharedLibraryElement.setAttributeNode(classLoaderDelegationAttribute);
        }

        if (this.version != null
                && !"".equals(this.version)) {
            final Attr versionAttribute = jbiDescriptorDocument
                    .createAttributeNS(XMLUtil.JBI_NAMESPACE, "version");
            versionAttribute.setValue(this.version);
            sharedLibraryElement.setAttributeNode(versionAttribute);
        }

        sharedLibraryElement.appendChild(this.identification.getNode(jbiDescriptorDocument));

        final Element sharedLibraryClassPathElement = jbiDescriptorDocument
                .createElementNS(XMLUtil.JBI_NAMESPACE,
                        "shared-library-class-path");
        for (int i = 0; i < this.sharedLibraryClassPath.size(); i++) {
            final Element pathElement = jbiDescriptorDocument.createElementNS(
                    XMLUtil.JBI_NAMESPACE, "path-element");
            pathElement.setTextContent(this.sharedLibraryClassPath.get(i));
            sharedLibraryClassPathElement.appendChild(pathElement);
        }
        sharedLibraryElement.appendChild(sharedLibraryClassPathElement);

        return sharedLibraryElement;
    }

    //
    // Getters and Setters
    //

    /**
     * Returns the class loading delegation model of this Shared Library. If
     * <code>null</code> the "parent-first" default delegation model must be
     * used.
     *
     * @return the delegation model: <code>"parent-first"</code> or
     * <code>"self-first"</code>.
     */
    public final String getClassLoaderDelegation() {
        return this.classLoaderDelegation;
    }

    /**
     * Returns component's identification data.
     *
     * @return the component's identification.
     */
    public final Identification getIdentification() {
        return this.identification;
    }

    /**
     * @return Returns the sharedLibraryClassPath.
     */
    public final List<String> getSharedLibraryClassPath() {
        return this.sharedLibraryClassPath;
    }

    /**
     * @return Returns the version.
     */
    public final String getVersion() {
        return this.version;
    }

    /**
     * Set the Shared Library class loading delegation model.
     *
     * @param delegationModel
     *            the delegation model to set: <code>"parent-first"</code> or
     *            <code>"self-first"</code>.
     */
    public final void setClassLoaderDelegation(final String delegationModel) {
        this.classLoaderDelegation = delegationModel;
    }

    /**
     * Set the component identification data.
     *
     * @param identification
     *            identification to set.
     */
    public final void setIdentification(final Identification identification) {
        this.identification = identification;
    }

    /**
     * @param sharedLibCP
     *            The sharedLibraryClassPath to set.
     */
    public final void setSharedLibraryClassPath(final List<String> sharedLibCP) {
        this.sharedLibraryClassPath = sharedLibCP;
    }

    /**
     * @param version
     *            The version to set.
     */
    public final void setVersion(final String version) {
        this.version = version;
    }
}
